##############################################################################
#
# Enfold Stage Deployment - Remote Deployment of Staged Content
# Copyright (C) 2005 Enfold Systems
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################
"""
$Id: patch.py 3078 2005-10-21 13:37:39Z sidnei $
"""

from Products.Archetypes.utils import wrap_method
from Products.EnSimpleStaging.content.stage import StagingArea
from Products.StageDeployment.event import publishLastChangedEvent
from Products.StageDeployment.event import publishStageEvent
from Products.StageDeployment.config import logger

def log(msg):
    logger.info(msg)

PATTERN = '__StageDeployment_%s__'
def call(self, __name__, *args, **kw):
    return getattr(self, PATTERN % __name__)(*args, **kw)

WRAPPER = '__stagedeployment_is_wrapper_method__'
ORIG_NAME = '__stagedeployment_original_method_name__'
def isWrapperMethod(meth):
    return getattr(meth, WRAPPER, False)

def wrap_method(klass, name, method, pattern=PATTERN):
    old_method = getattr(klass, name)
    if isWrapperMethod(old_method):
        # Double-wrapping considered harmful.
        log('Already wrapped method %s.%s, skipping.' %
            (klass.__name__, name))
        return
    log('Wrapping method %s.%s' % (klass.__name__, name))
    new_name = pattern % name
    setattr(klass, new_name, old_method)
    setattr(method, ORIG_NAME, new_name)
    setattr(method, WRAPPER, True)
    setattr(klass, name, method)

def unwrap_method(klass, name):
    old_method = getattr(klass, name)
    if not isWrapperMethod(old_method):
        raise ValueError, ('Trying to unwrap non-wrapped '
                           'method %s.%s' % (klass.__name__, name))
    orig_name = getattr(old_method, ORIG_NAME)
    new_method = getattr(klass, orig_name)
    delattr(klass, orig_name)
    setattr(klass, name, new_method)

def publishLastChanged(self, *args, **kw):
    ret = call(self, 'publishLastChanged', *args, **kw)
    publishLastChangedEvent(self)
    return ret

def publishStage(self, *args, **kw):
    ret = call(self, 'publishStage', *args, **kw)
    publishStageEvent(self)
    return ret

wrap_method(StagingArea, 'publishLastChanged',
            publishLastChanged, pattern=PATTERN)

wrap_method(StagingArea, 'publishStage',
            publishStage, pattern=PATTERN)
